:
# NAME:
#	cpCard - copy pics via USB card reader
#
# SYNOPSIS:
#	cpCard ["options"] "dest"
#	cpSD ["options"] "dest"
#	cpCF ["options"] "dest"
#
# DESCRIPTION:
#	This script mounts a card reader device and copies files into
#	"dest".  Since the script must be run as root, the ownership
#	of files is determined by the top level pics directory (see
#	options).
#
#	Options:
#
#	-n	Do nothing
#
#	-v	Be verbose
#
#	-d "dev"
#		Use "dev" rather than anything we might glean from
#		the dmesg output.
#
#	-t "type"
#		Specify the type of device we are reading from.
#		If $0 isn't cpCard, we assume the bit after cp is a
#		default for "type".
#
#	-e "expr"
#		Sometimes "type" isn't enough clue.
#	
#	-p "pics"
#		Identify the top level directory for the pics
#		collection.  If "dest" isn't an absolute dir, it is
#		assumed to be relative to "pics".  The owner:group for
#		"pics" is used for all new files and dirs.
#
#	-r "user"
#		Run mkdir and cp commands as "user" rather than the
#		owner of "pics".
#
# BUGS:
#	Mounting the card reader is inherently OS specific, this
#	script probably only works on NetBSD.
#			
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>
#

# RCSid:
#	$Id: cpCard.sh,v 1.4 2006/04/06 03:52:06 sjg Exp $
#
#	@(#) Copyright (c) 2006 Simon J. Gerraty
#
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to copy, redistribute or otherwise
#	use this file is hereby granted provided that 
#	the above copyright notice and this notice are
#	left intact. 
#      
#	Please send copies of changes and bug-fixes to:
#	sjg@crufty.net
#

MYNAME=cpCard
Myname=`basename $0 .sh`
Mydir=`dirname $0`

# the options
opt_str=t:d:e:nvr:m:p:

case $Myname in
$MYNAME) ;;
cp*)
        opt_t=`echo $Myname | sed 's,^cp,,'`
        ;;
*)
        opt_t=CF
        ;;
esac

Which() {
	case $1 in
	-?) _t=$1; shift;;
	*) _t=-x;;
	esac
	f=$1; shift
	dirs=`IFS=:; echo ${*:-$PATH}`
	for d in "" $dirs
	do
		test $_t $d/$f || continue
		echo ${d:+$d/}$f
		break
	done
}

ECHO=:
SKIP=
opt_p=/pics
opt_m=/mnt
opt_r=
# set this now so it can be reset by eval_args
RUNAS=`Which runas`

. $Mydir/eval_args.sh

case "$opt_n" in
""|0) ;;
*) SKIP=:; ECHO=echo;;
esac
case "$opt_v" in
""|0) ;;
*) ECHO=echo;;
esac

RunAs() {
	_user=$1; shift
        
	if [ "$RUNAS" ]; then
		ENV= $RUNAS $_user $*
	else
		echo "$*" | ENV= ${SU:-su} $_user
	fi
}

error() {
        echo "ERROR: $@" >&2
        exit 1
}

get_dev() {
        case "$opt_t" in
	CF) re="${opt_e:-CompactFlash}";;
	*) re="${opt_e:-$opt_t}";;
	esac
        egrep "^sd.*USB.*$re" /var/run/dmesg.boot 2> /dev/null |
        (read dev rest; echo $dev)
}

Do() {
        $ECHO "$@"
        $SKIP "$@"
}

CP() {
     dest=$1; shift
     RunAs $opt_r cp -p $* $dest
     for f in $*
     do
	echo $f
     done | sed "s,.*/,$dest/," | RunAs $opt_r xargs chmod 444
}

case "$1" in
"") error No destination specified;;
/*) dest=$1;;
*) dest=$opt_p/$1;;
esac

dev=${opt_d:-`get_dev`}
case "$dev:$opt_t" in
:CF)	dev=wd1;;		# my laptop with PCMCIA adapter
esac
dev=${opt_d:-$dev}
[ "$dev" ] || error No device found or specified for $opt_t
Do disklabel $dev
# NetBSD sees the MSDOS partition as 'e'
Do mount -t msdos /dev/${dev}e $opt_m || exit 1
Do mount $opt_p || exit 1
umask 002
case "$opt_r" in
"")	opt_r=`'ls' -ld $opt_p | (read perm blk owner rest; echo $owner)`
        ;;
esac
Do RunAs $opt_r mkdir -p $dest
umask 0222
if exists -d $opt_m/dcim/*canon; then
	Do CP $dest $opt_m/dcim/*canon/*_*
elif exists -d $opt_m/dcim/*nikon; then
	Do CP $dest $opt_m/dcim/*nikon/dscn*
fi
sync; sleep 1; sync
Do umount $opt_m
Do umount $opt_p
